==> cvcapture-0.1.0.tgz: cvcapture-0.1.0/cvcapture.c <==
/*
   +----------------------------------------------------------------------+
   | All rights reserved                                                  |
   |                                                                      |
   | Redistribution and use in source and binary forms, with or without   |
   | modification, are permitted provided that the following conditions   |
   | are met:                                                             |
   |                                                                      |
   | 1. Redistributions of source code must retain the above copyright    |
   |    notice, this list of conditions and the following disclaimer.     |
   | 2. Redistributions in binary form must reproduce the above copyright |
   |    notice, this list of conditions and the following disclaimer in   |
   |    the documentation and/or other materials provided with the        |
   |    distribution.                                                     |
   | 3. The names of the authors may not be used to endorse or promote    |
   |    products derived from this software without specific prior        |
   |    written permission.                                               |
   |                                                                      |
   | THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS  |
   | "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    |
   | LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS    |
   | FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE       |
   | COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,  |
   | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, |
   | BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;     |
   | LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     |
   | CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT   |
   | LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN    | 
   | ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE      |
   | POSSIBILITY OF SUCH DAMAGE.                                          |
   +----------------------------------------------------------------------+
   | Authors: Ryusuke Sekiyama <rsky0711@gmail.com>                       |
   +----------------------------------------------------------------------+
*/

/* $ Id: $ */ 

#include "php_cvcapture.h"

#if HAVE_CVCAPTURE

/* {{{ cvcapture_functions[] */
function_entry cvcapture_functions[] = {
	PHP_FE(cv_camera_capture   , cv_camera_capture_arg_info)
	PHP_FE(cv_file_capture     , cv_file_capture_arg_info)
	{ NULL, NULL, NULL }
};
/* }}} */


/* {{{ cvcapture_module_entry
 */
zend_module_entry cvcapture_module_entry = {
	STANDARD_MODULE_HEADER,
	"cvcapture",
	cvcapture_functions,
	PHP_MINIT(cvcapture),     /* Replace with NULL if there is nothing to do at php startup   */ 
	PHP_MSHUTDOWN(cvcapture), /* Replace with NULL if there is nothing to do at php shutdown  */
	PHP_RINIT(cvcapture),     /* Replace with NULL if there is nothing to do at request start */
	PHP_RSHUTDOWN(cvcapture), /* Replace with NULL if there is nothing to do at request end   */
	PHP_MINFO(cvcapture),
	"0.1.0", 
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_CVCAPTURE
ZEND_GET_MODULE(cvcapture)
#endif


/* {{{ PHP_MINIT_FUNCTION */
PHP_MINIT_FUNCTION(cvcapture)
{
	REGISTER_LONG_CONSTANT("CV_CAP_ANY", CV_CAP_ANY, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_MIL", CV_CAP_MIL, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_VFW", CV_CAP_VFW, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_V4L", CV_CAP_V4L, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_V4L2", CV_CAP_V4L2, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_FIREWARE", CV_CAP_FIREWARE, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_IEEE1394", CV_CAP_IEEE1394, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_DC1394", CV_CAP_DC1394, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_CMU1394", CV_CAP_FIREWARE, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_STEREO", CV_CAP_STEREO, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_TYZX", CV_CAP_TYZX, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_LEFT", CV_TYZX_LEFT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_RIGHT", CV_TYZX_RIGHT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_COLOR", CV_TYZX_COLOR, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_Z", CV_TYZX_Z, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_QT", CV_CAP_QT, CONST_PERSISTENT | CONST_CS);

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MSHUTDOWN_FUNCTION */
PHP_MSHUTDOWN_FUNCTION(cvcapture)
{

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RINIT_FUNCTION */
PHP_RINIT_FUNCTION(cvcapture)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RSHUTDOWN_FUNCTION */
PHP_RSHUTDOWN_FUNCTION(cvcapture)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MINFO_FUNCTION */
PHP_MINFO_FUNCTION(cvcapture)
{
	php_info_print_box_start(0);
	php_printf("<p>Capture frames from webcams.</p>\n");
	php_printf("<p>Version 0.1.0alpha (2008-02-04)</p>\n");
	php_printf("<p><b>Authors:</b></p>\n");
	php_printf("<p>Ryusuke Sekiyama &lt;rsky0711@gmail.com&gt; (lead)</p>\n");
	php_info_print_box_end();
	/* add your stuff here */

}
/* }}} */


/* {{{ proto bool cv_camera_capture(string filename[, int index[, mixed &size]])
  Capture from camera. */
PHP_FUNCTION(cv_camera_capture)
{
	const char * filename = NULL;
	int filename_len = 0;
	long index = CV_CAP_ANY;
	zval * size = NULL;
	char *fullpath;
	CvCapture *capture;
	IplImage *image;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lz", &filename, &filename_len, &index, &size) == FAILURE) {
		return;
	}

	if (strlen(filename) != filename_len ||
		(fullpath = expand_filepath(filename, NULL TSRMLS_CC)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Wrong filename given");
		RETURN_FALSE;
	}

	if (php_check_open_basedir(fullpath TSRMLS_CC) ||
		(PG(safe_mode) && !php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR)))
	{
		efree(fullpath);
		RETURN_FALSE;
	}

	capture = cvCreateCameraCapture((int)index);
	if (capture == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create camera capture");
		efree(fullpath);
		RETURN_FALSE;
	}

	image = cvQueryFrame(capture);
	if (image == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot retrieve image");
		cvReleaseCapture(&capture);
		efree(fullpath);
		RETURN_FALSE;
	}

	if (!cvSaveImage(fullpath, image)) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot save image to '%s'", filename);
		cvReleaseCapture(&capture);
		efree(fullpath);
		RETURN_FALSE;
	}

	if (size != NULL) {
		CvSize imgsize = cvGetSize(image);
		zval_dtor(size);
		array_init(size);
		add_next_index_long(size, (long)imgsize.width);
		add_next_index_long(size, (long)imgsize.height);
	}

	cvReleaseCapture(&capture);
	efree(fullpath);
	RETURN_TRUE;
}
/* }}} cv_camera_capture */


/* {{{ proto bool cv_file_capture(string dst_filename, string src_filename[, mixed &size])
  Capture from video file. */
PHP_FUNCTION(cv_file_capture)
{
	const char * dst_filename = NULL;
	int dst_filename_len = 0;
	const char * src_filename = NULL;
	int src_filename_len = 0;
	zval * size = NULL;
	char *dst_fullpath, *src_fullpath;
	CvCapture *capture;
	IplImage *image;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|z", &dst_filename, &dst_filename_len, &src_filename, &src_filename_len, &size) == FAILURE) {
		return;
	}

	if (strlen(dst_filename) != dst_filename_len ||
		(dst_fullpath = expand_filepath(dst_filename, NULL TSRMLS_CC)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Wrong destination filename given");
		RETURN_FALSE;
	}

	if (strlen(src_filename) != src_filename_len ||
		(src_fullpath = expand_filepath(src_filename, NULL TSRMLS_CC)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Wrong source filename given");
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	if (php_check_open_basedir(dst_fullpath TSRMLS_CC) ||
		php_check_open_basedir(src_fullpath TSRMLS_CC) ||
		(PG(safe_mode) && (!php_checkuid(dst_fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR) ||
			!php_checkuid(src_fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR))))
	{
		efree(src_fullpath);
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	capture = cvCreateFileCapture(src_fullpath);
	if (capture == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create file capture from '%s'", src_fullpath);
		efree(src_fullpath);
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	image = cvQueryFrame(capture);
	if (image == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot retrieve image");
		cvReleaseCapture(&capture);
		efree(src_fullpath);
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	if (!cvSaveImage(dst_fullpath, image)) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot save image to '%s'", dst_fullpath);
		cvReleaseCapture(&capture);
		efree(src_fullpath);
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	if (size != NULL) {
		CvSize imgsize = cvGetSize(image);
		zval_dtor(size);
		array_init(size);
		add_next_index_long(size, (long)imgsize.width);
		add_next_index_long(size, (long)imgsize.height);
	}

	cvReleaseCapture(&capture);
	efree(src_fullpath);
	efree(dst_fullpath);
	RETURN_TRUE;
}
/* }}} cv_file_capture */

#endif /* HAVE_CVCAPTURE */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */

==> cvcapture-0.2.0.tgz: cvcapture-0.2.0/cvcapture.c <==
/*
   +----------------------------------------------------------------------+
   | All rights reserved                                                  |
   |                                                                      |
   | Redistribution and use in source and binary forms, with or without   |
   | modification, are permitted provided that the following conditions   |
   | are met:                                                             |
   |                                                                      |
   | 1. Redistributions of source code must retain the above copyright    |
   |    notice, this list of conditions and the following disclaimer.     |
   | 2. Redistributions in binary form must reproduce the above copyright |
   |    notice, this list of conditions and the following disclaimer in   |
   |    the documentation and/or other materials provided with the        |
   |    distribution.                                                     |
   | 3. The names of the authors may not be used to endorse or promote    |
   |    products derived from this software without specific prior        |
   |    written permission.                                               |
   |                                                                      |
   | THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS  |
   | "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    |
   | LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS    |
   | FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE       |
   | COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,  |
   | INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, |
   | BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;     |
   | LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER     |
   | CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT   |
   | LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN    | 
   | ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE      |
   | POSSIBILITY OF SUCH DAMAGE.                                          |
   +----------------------------------------------------------------------+
   | Authors: Ryusuke Sekiyama <rsky0711@gmail.com>                       |
   +----------------------------------------------------------------------+
*/

/* $ Id: $ */ 

#include "php_cvcapture.h"

#if HAVE_CVCAPTURE

/* {{{ Resource destructors */
int le_cvcapture;
void cvcapture_dtor(zend_rsrc_list_entry *rsrc TSRMLS_DC)
{
	CvCapture * resource = (CvCapture *)(rsrc->ptr);

	do {
		cvReleaseCapture(&resource);
	} while (0);
}

/* }}} *

/* {{{ cvcapture_functions[] */
function_entry cvcapture_functions[] = {
	PHP_FE(cv_camera_capture   , cv_camera_capture_arg_info)
	PHP_FE(cv_file_capture     , cv_file_capture_arg_info)
	PHP_FE(cv_create_camera_capture, cv_create_camera_capture_arg_info)
	PHP_FE(cv_create_file_capture, cv_create_file_capture_arg_info)
	PHP_FE(cv_save_capture     , cv_save_capture_arg_info)
	{ NULL, NULL, NULL }
};
/* }}} */


/* {{{ cvcapture_module_entry
 */
zend_module_entry cvcapture_module_entry = {
	STANDARD_MODULE_HEADER,
	"cvcapture",
	cvcapture_functions,
	PHP_MINIT(cvcapture),     /* Replace with NULL if there is nothing to do at php startup   */ 
	PHP_MSHUTDOWN(cvcapture), /* Replace with NULL if there is nothing to do at php shutdown  */
	PHP_RINIT(cvcapture),     /* Replace with NULL if there is nothing to do at request start */
	PHP_RSHUTDOWN(cvcapture), /* Replace with NULL if there is nothing to do at request end   */
	PHP_MINFO(cvcapture),
	"0.2.0", 
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_CVCAPTURE
ZEND_GET_MODULE(cvcapture)
#endif


/* {{{ PHP_MINIT_FUNCTION */
PHP_MINIT_FUNCTION(cvcapture)
{
	REGISTER_LONG_CONSTANT("CV_CAP_ANY", CV_CAP_ANY, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_MIL", CV_CAP_MIL, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_VFW", CV_CAP_VFW, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_V4L", CV_CAP_V4L, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_V4L2", CV_CAP_V4L2, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_FIREWARE", CV_CAP_FIREWARE, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_IEEE1394", CV_CAP_IEEE1394, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_DC1394", CV_CAP_DC1394, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_CMU1394", CV_CAP_FIREWARE, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_STEREO", CV_CAP_STEREO, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_TYZX", CV_CAP_TYZX, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_LEFT", CV_TYZX_LEFT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_RIGHT", CV_TYZX_RIGHT, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_COLOR", CV_TYZX_COLOR, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_TYZX_Z", CV_TYZX_Z, CONST_PERSISTENT | CONST_CS);
	REGISTER_LONG_CONSTANT("CV_CAP_QT", CV_CAP_QT, CONST_PERSISTENT | CONST_CS);
	le_cvcapture = zend_register_list_destructors_ex(cvcapture_dtor, 
						   NULL, "cvcapture", module_number);

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MSHUTDOWN_FUNCTION */
PHP_MSHUTDOWN_FUNCTION(cvcapture)
{

	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RINIT_FUNCTION */
PHP_RINIT_FUNCTION(cvcapture)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_RSHUTDOWN_FUNCTION */
PHP_RSHUTDOWN_FUNCTION(cvcapture)
{
	/* add your stuff here */

	return SUCCESS;
}
/* }}} */


/* {{{ PHP_MINFO_FUNCTION */
PHP_MINFO_FUNCTION(cvcapture)
{
	php_info_print_box_start(0);
	php_printf("<p>Capture frames from webcams.</p>\n");
	php_printf("<p>Version 0.2.0beta (2008-03-03)</p>\n");
	php_printf("<p><b>Authors:</b></p>\n");
	php_printf("<p>Ryusuke Sekiyama &lt;rsky0711@gmail.com&gt; (lead)</p>\n");
	php_info_print_box_end();
	/* add your stuff here */

}
/* }}} */


/* {{{ proto bool cv_camera_capture(string filename[, int index[, mixed &size]])
  Capture from camera. */
PHP_FUNCTION(cv_camera_capture)
{
	const char * filename = NULL;
	int filename_len = 0;
	long index = CV_CAP_ANY;
	zval * size = NULL;
	char *fullpath;
	CvCapture *capture;
	IplImage *image;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s|lz", &filename, &filename_len, &index, &size) == FAILURE) {
		return;
	}

	if (strlen(filename) != filename_len ||
		(fullpath = expand_filepath(filename, NULL TSRMLS_CC)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Wrong filename given");
		RETURN_FALSE;
	}

	if (php_check_open_basedir(fullpath TSRMLS_CC) ||
		(PG(safe_mode) && !php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR)))
	{
		efree(fullpath);
		RETURN_FALSE;
	}

	capture = cvCreateCameraCapture((int)index);
	if (capture == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create camera capture");
		efree(fullpath);
		RETURN_FALSE;
	}

	image = cvQueryFrame(capture);
	if (image == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot retrieve image");
		cvReleaseCapture(&capture);
		efree(fullpath);
		RETURN_FALSE;
	}

	if (!cvSaveImage(fullpath, image)) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot save image to '%s'", filename);
		cvReleaseCapture(&capture);
		efree(fullpath);
		RETURN_FALSE;
	}

	if (size != NULL) {
		CvSize imgsize = cvGetSize(image);
		zval_dtor(size);
		array_init(size);
		add_next_index_long(size, (long)imgsize.width);
		add_next_index_long(size, (long)imgsize.height);
	}

	cvReleaseCapture(&capture);
	efree(fullpath);
	RETURN_TRUE;
}
/* }}} cv_camera_capture */


/* {{{ proto bool cv_file_capture(string dst_filename, string src_filename[, mixed &size])
  Capture from video file. */
PHP_FUNCTION(cv_file_capture)
{
	const char * dst_filename = NULL;
	int dst_filename_len = 0;
	const char * src_filename = NULL;
	int src_filename_len = 0;
	zval * size = NULL;
	char *dst_fullpath, *src_fullpath;
	CvCapture *capture;
	IplImage *image;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss|z", &dst_filename, &dst_filename_len, &src_filename, &src_filename_len, &size) == FAILURE) {
		return;
	}

	if (strlen(dst_filename) != dst_filename_len ||
		(dst_fullpath = expand_filepath(dst_filename, NULL TSRMLS_CC)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Wrong destination filename given");
		RETURN_FALSE;
	}

	if (strlen(src_filename) != src_filename_len ||
		(src_fullpath = expand_filepath(src_filename, NULL TSRMLS_CC)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Wrong source filename given");
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	if (php_check_open_basedir(dst_fullpath TSRMLS_CC) ||
		php_check_open_basedir(src_fullpath TSRMLS_CC) ||
		(PG(safe_mode) && (!php_checkuid(dst_fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR) ||
			!php_checkuid(src_fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR))))
	{
		efree(src_fullpath);
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	capture = cvCreateFileCapture(src_fullpath);
	if (capture == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create file capture from '%s'", src_fullpath);
		efree(src_fullpath);
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	image = cvQueryFrame(capture);
	if (image == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot retrieve image");
		cvReleaseCapture(&capture);
		efree(src_fullpath);
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	if (!cvSaveImage(dst_fullpath, image)) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot save image to '%s'", dst_fullpath);
		cvReleaseCapture(&capture);
		efree(src_fullpath);
		efree(dst_fullpath);
		RETURN_FALSE;
	}

	if (size != NULL) {
		CvSize imgsize = cvGetSize(image);
		zval_dtor(size);
		array_init(size);
		add_next_index_long(size, (long)imgsize.width);
		add_next_index_long(size, (long)imgsize.height);
	}

	cvReleaseCapture(&capture);
	efree(src_fullpath);
	efree(dst_fullpath);
	RETURN_TRUE;
}
/* }}} cv_file_capture */


/* {{{ proto resource cvcapture cv_create_camera_capture([int index])
  Start capturing frames from camera. */
PHP_FUNCTION(cv_create_camera_capture)
{
	CvCapture * return_res;
	long return_res_id = -1;

	long index = CV_CAP_ANY;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|l", &index) == FAILURE) {
		return;
	}

	return_res = cvCreateCameraCapture((int)index);
	if (return_res == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create camera capture");
		RETURN_FALSE;
	}

	ZEND_REGISTER_RESOURCE(return_value, return_res, le_cvcapture);
}
/* }}} cv_create_camera_capture */


/* {{{ proto resource cvcapture cv_create_file_capture(string filename)
  Start capturing frames from video file. */
PHP_FUNCTION(cv_create_file_capture)
{
	CvCapture * return_res;
	long return_res_id = -1;

	const char * filename = NULL;
	int filename_len = 0;
	char *fullpath;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &filename, &filename_len) == FAILURE) {
		return;
	}

	if (strlen(filename) != filename_len ||
		(fullpath = expand_filepath(filename, NULL TSRMLS_CC)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Wrong filename given");
		RETURN_FALSE;
	}

	if (php_check_open_basedir(fullpath TSRMLS_CC) ||
		(PG(safe_mode) && !php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR)))
	{
		efree(fullpath);
		RETURN_FALSE;
	}

	return_res = cvCreateFileCapture(fullpath);
	if (return_res == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot create file capture from '%s'", fullpath);
		efree(fullpath);
		RETURN_FALSE;
	}

	efree(fullpath);
	ZEND_REGISTER_RESOURCE(return_value, return_res, le_cvcapture);
}
/* }}} cv_create_file_capture */


/* {{{ proto bool cv_save_capture(resource cvcapture capture, string filename[, mixed &size])
  Capture a frame. */
PHP_FUNCTION(cv_save_capture)
{
	zval * capture = NULL;
	int capture_id = -1;
	CvCapture * res_capture;

	const char * filename = NULL;
	int filename_len = 0;
	zval * size = NULL;
	char *fullpath;
	IplImage *image;

	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rs|z", &capture, &filename, &filename_len, &size) == FAILURE) {
		return;
	}
	ZEND_FETCH_RESOURCE(res_capture, CvCapture *, &capture, capture_id, "cvcapture", le_cvcapture);

	if (strlen(filename) != filename_len ||
		(fullpath = expand_filepath(filename, NULL TSRMLS_CC)) == NULL)
	{
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Wrong filename given");
		RETURN_FALSE;
	}

	if (php_check_open_basedir(fullpath TSRMLS_CC) ||
		(PG(safe_mode) && !php_checkuid(fullpath, NULL, CHECKUID_CHECK_FILE_AND_DIR)))
	{
		efree(fullpath);
		RETURN_FALSE;
	}

	image = cvQueryFrame(res_capture);
	if (image == NULL) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot retrieve image");
		efree(fullpath);
		RETURN_FALSE;
	}

	if (!cvSaveImage(fullpath, image)) {
		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot save image to '%s'", filename);
		efree(fullpath);
		RETURN_FALSE;
	}

	if (size != NULL) {
		CvSize imgsize = cvGetSize(image);
		zval_dtor(size);
		array_init(size);
		add_next_index_long(size, (long)imgsize.width);
		add_next_index_long(size, (long)imgsize.height);
	}

	efree(fullpath);
	RETURN_TRUE;
}
/* }}} cv_save_capture */

#endif /* HAVE_CVCAPTURE */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
